# coding: utf-8

"""
    Fabric Credential Manager API

    This is Fabric Credential Manager API  # noqa: E501

    OpenAPI spec version: 1.0.2
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class JwksKeys(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'kty': 'str',
        'e': 'str',
        'n': 'str',
        'use': 'str',
        'alg': 'str',
        'kid': 'str'
    }

    attribute_map = {
        'kty': 'kty',
        'e': 'e',
        'n': 'n',
        'use': 'use',
        'alg': 'alg',
        'kid': 'kid'
    }

    def __init__(self, kty=None, e=None, n=None, use=None, alg=None, kid=None):  # noqa: E501
        """JwksKeys - a model defined in Swagger"""  # noqa: E501
        self._kty = None
        self._e = None
        self._n = None
        self._use = None
        self._alg = None
        self._kid = None
        self.discriminator = None
        if kty is not None:
            self.kty = kty
        if e is not None:
            self.e = e
        if n is not None:
            self.n = n
        if use is not None:
            self.use = use
        if alg is not None:
            self.alg = alg
        if kid is not None:
            self.kid = kid

    @property
    def kty(self):
        """Gets the kty of this JwksKeys.  # noqa: E501

        Key Type  # noqa: E501

        :return: The kty of this JwksKeys.  # noqa: E501
        :rtype: str
        """
        return self._kty

    @kty.setter
    def kty(self, kty):
        """Sets the kty of this JwksKeys.

        Key Type  # noqa: E501

        :param kty: The kty of this JwksKeys.  # noqa: E501
        :type: str
        """

        self._kty = kty

    @property
    def e(self):
        """Gets the e of this JwksKeys.  # noqa: E501

        Exponent Parameter  # noqa: E501

        :return: The e of this JwksKeys.  # noqa: E501
        :rtype: str
        """
        return self._e

    @e.setter
    def e(self, e):
        """Sets the e of this JwksKeys.

        Exponent Parameter  # noqa: E501

        :param e: The e of this JwksKeys.  # noqa: E501
        :type: str
        """

        self._e = e

    @property
    def n(self):
        """Gets the n of this JwksKeys.  # noqa: E501

        Modulus Parameter  # noqa: E501

        :return: The n of this JwksKeys.  # noqa: E501
        :rtype: str
        """
        return self._n

    @n.setter
    def n(self, n):
        """Sets the n of this JwksKeys.

        Modulus Parameter  # noqa: E501

        :param n: The n of this JwksKeys.  # noqa: E501
        :type: str
        """

        self._n = n

    @property
    def use(self):
        """Gets the use of this JwksKeys.  # noqa: E501

        Public Key Use Parameter  # noqa: E501

        :return: The use of this JwksKeys.  # noqa: E501
        :rtype: str
        """
        return self._use

    @use.setter
    def use(self, use):
        """Sets the use of this JwksKeys.

        Public Key Use Parameter  # noqa: E501

        :param use: The use of this JwksKeys.  # noqa: E501
        :type: str
        """

        self._use = use

    @property
    def alg(self):
        """Gets the alg of this JwksKeys.  # noqa: E501

        Algorithm Parameter  # noqa: E501

        :return: The alg of this JwksKeys.  # noqa: E501
        :rtype: str
        """
        return self._alg

    @alg.setter
    def alg(self, alg):
        """Sets the alg of this JwksKeys.

        Algorithm Parameter  # noqa: E501

        :param alg: The alg of this JwksKeys.  # noqa: E501
        :type: str
        """

        self._alg = alg

    @property
    def kid(self):
        """Gets the kid of this JwksKeys.  # noqa: E501

        Key Id Header Parameter  # noqa: E501

        :return: The kid of this JwksKeys.  # noqa: E501
        :rtype: str
        """
        return self._kid

    @kid.setter
    def kid(self, kid):
        """Sets the kid of this JwksKeys.

        Key Id Header Parameter  # noqa: E501

        :param kid: The kid of this JwksKeys.  # noqa: E501
        :type: str
        """

        self._kid = kid

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(JwksKeys, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, JwksKeys):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import logging
import time
from typing import List

from selenium import webdriver


class SessionHelper:
    def __init__(self, *, url: str = "https://cm.fabric-testbed.net", cookie_name: str = "fabric-service",
                 wait_timeout: int = 500, wait_interval: int = 5):
        self.logger = logging.getLogger()
        self.url = url
        self.cookie_name = cookie_name
        self.wait_timeout = wait_timeout
        self.wait_interval = wait_interval

    def __extract_fabric_cookie(self, *, cookies: List[dict]):
        ret_val = None
        for c in cookies:
            name = c.get('name')
            if ret_val is None and name == self.cookie_name:
                ret_val = c.get('value')
                ret_val = f"{name}={ret_val}"
                break
        self.logger.debug(f"Returning cookie: {ret_val}")
        return ret_val

    def login(self, browser_name: str = "chrome"):
        if browser_name.lower() == "firefox":
            browser = webdriver.Firefox()
        elif browser_name.lower() == "safari":
            browser = webdriver.Safari()
        elif browser_name.lower() == "edge":
            browser = webdriver.Edge()
        else:
            browser = webdriver.Chrome()
        browser.get(self.url)

        # Get the cookies
        cookie = self.__extract_fabric_cookie(cookies=browser.get_cookies())
        start = time.time()
        while cookie is None or time.time() > start + self.wait_timeout:
            time.sleep(self.wait_interval)
            self.logger.info("Checking if the user has logged in!")
            cookie = self.__extract_fabric_cookie(cookies=browser.get_cookies())

        # Print the cookies
        if cookie is None:
            msg = f"Timeout occurred - User did not login within {self.wait_timeout} or {self.cookie_name} not found"
            self.logger.error(msg)
            raise Exception(msg)

        # Close the browser
        browser.quit()

        #return requests.cookies.create_cookie(name=self.cookie_name, value=cookie)
        return cookie



# coding: utf-8

"""
    Fabric Credential Manager API

    This is Fabric Credential Manager API  # noqa: E501

    OpenAPI spec version: 1.0.2
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import swagger_client
from swagger_client.models.revoke_list import RevokeList  # noqa: E501
from swagger_client.rest import ApiException


class TestRevokeList(unittest.TestCase):
    """RevokeList unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testRevokeList(self):
        """Test RevokeList"""
        # FIXME: construct object with mandatory attributes with example values
        # model = swagger_client.models.revoke_list.RevokeList()  # noqa: E501
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

# flake8: noqa
"""
    Fabric Credential Manager API

    This is Fabric Credential Manager API  # noqa: E501

    OpenAPI spec version: 1.0.2
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

# import models into model package
from fabric_cm.credmgr.swagger_client.models.decoded_token import DecodedToken
from fabric_cm.credmgr.swagger_client.models.jwks import Jwks
from fabric_cm.credmgr.swagger_client.models.jwks_keys import JwksKeys
from fabric_cm.credmgr.swagger_client.models.request import Request
from fabric_cm.credmgr.swagger_client.models.revoke_list import RevokeList
from fabric_cm.credmgr.swagger_client.models.status200_ok_no_content import Status200OkNoContent
from fabric_cm.credmgr.swagger_client.models.status200_ok_no_content_data import Status200OkNoContentData
from fabric_cm.credmgr.swagger_client.models.status200_ok_paginated import Status200OkPaginated
from fabric_cm.credmgr.swagger_client.models.status200_ok_single import Status200OkSingle
from fabric_cm.credmgr.swagger_client.models.status400_bad_request import Status400BadRequest
from fabric_cm.credmgr.swagger_client.models.status400_bad_request_errors import Status400BadRequestErrors
from fabric_cm.credmgr.swagger_client.models.status401_unauthorized import Status401Unauthorized
from fabric_cm.credmgr.swagger_client.models.status401_unauthorized_errors import Status401UnauthorizedErrors
from fabric_cm.credmgr.swagger_client.models.status403_forbidden import Status403Forbidden
from fabric_cm.credmgr.swagger_client.models.status403_forbidden_errors import Status403ForbiddenErrors
from fabric_cm.credmgr.swagger_client.models.status404_not_found import Status404NotFound
from fabric_cm.credmgr.swagger_client.models.status404_not_found_errors import Status404NotFoundErrors
from fabric_cm.credmgr.swagger_client.models.status500_internal_server_error import Status500InternalServerError
from fabric_cm.credmgr.swagger_client.models.status500_internal_server_error_errors import Status500InternalServerErrorErrors
from fabric_cm.credmgr.swagger_client.models.token import Token
from fabric_cm.credmgr.swagger_client.models.token_post import TokenPost
from fabric_cm.credmgr.swagger_client.models.tokens import Tokens
from fabric_cm.credmgr.swagger_client.models.version import Version
from fabric_cm.credmgr.swagger_client.models.version_data import VersionData



# coding: utf-8

"""
    Fabric Credential Manager API

    This is Fabric Credential Manager API  # noqa: E501

    OpenAPI spec version: 1.0.2
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

from fabric_cm.credmgr.swagger_client.api.default_api import DefaultApi  # noqa: E501
from fabric_cm.credmgr.swagger_client.rest import ApiException


class TestDefaultApi(unittest.TestCase):
    """DefaultApi unit test stubs"""

    def setUp(self):
        self.api = DefaultApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_certs_get(self):
        """Test case for certs_get

        Return Public Keys to verify signature of the tokens  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import enum
import json
from datetime import datetime
from typing import Tuple, Any, List, Union

from atomicwrites import atomic_write

from fabric_cm.credmgr import swagger_client
from fabric_cm.credmgr.session_helper import SessionHelper
from fabric_cm.credmgr.swagger_client import Token, TokenPost
from fabric_cm.credmgr.swagger_client.models import DecodedToken
from fabric_cm.credmgr.swagger_client.rest import ApiException as CredMgrException


@enum.unique
class Status(enum.Enum):
    OK = 1
    INVALID_ARGUMENTS = 2
    FAILURE = 3

    def interpret(self, exception=None):
        interpretations = {
            1: "Success",
            2: "Invalid Arguments",
            3: "Failure"
          }
        if exception is None:
            return interpretations[self.value]
        else:
            return str(exception) + ". " + interpretations[self.value]


class TokenState(enum.Enum):
    Nascent = enum.auto(),
    Valid = enum.auto(),
    Refreshed = enum.auto(),
    Revoked = enum.auto(),
    Expired = enum.auto(),

    def __str__(self):
        return self.name

    def __repr__(self):
        return self.name

    @staticmethod
    def state_from_str(state: str):
        if state is None:
            return state

        for t in TokenState:
            if state == str(t):
                return t

        return None

    @staticmethod
    def state_list_to_str_list(states: list):
        if states is None:
            result = []

            for t in TokenState:
                result.append(str(t))
            return result

        result = []
        for t in states:
            result.append(str(t))

        return result


class TokenType(enum.Enum):
    Identity = enum.auto(),
    Refresh = enum.auto()

    def __str__(self):
        return self.name

    def __repr__(self):
        return self.name


class CredmgrProxy:
    """
    Credential Manager Proxy
    """
    ID_TOKEN = "id_token"
    REFRESH_TOKEN = "refresh_token"
    TIME_FORMAT = "%Y-%m-%d %H:%M:%S %z"
    CREATED_AT = "created_at"
    ERROR = "error"
    PROP_AUTHORIZATION = 'Authorization'
    PROP_BEARER = 'Bearer'

    def __init__(self, credmgr_host: str, cookie_name: str = "fabric-service",
                 wait_timeout: int = 500, wait_interval: int = 5):
        self.host = credmgr_host
        self.cookie_name = cookie_name
        self.wait_timeout = wait_timeout
        self.wait_interval = wait_interval
        self.tokens_api = None
        if credmgr_host is not None:
            # create an instance of the API class
            configuration = swagger_client.configuration.Configuration()
            configuration.host = f"https://{credmgr_host}/credmgr/"
            api_instance = swagger_client.ApiClient(configuration)
            self.tokens_api = swagger_client.TokensApi(api_client=api_instance)
            self.default_api = swagger_client.DefaultApi(api_client=api_instance)
            self.version_api = swagger_client.VersionApi(api_client=api_instance)

    def __set_tokens(self, *, token: str):
        """
        Set tokens
        @param token token
        """
        # Set the tokens
        self.tokens_api.api_client.configuration.api_key[self.PROP_AUTHORIZATION] = token
        self.tokens_api.api_client.configuration.api_key_prefix[self.PROP_AUTHORIZATION] = self.PROP_BEARER

    def create(self, scope: str = "all", project_id: str = None, project_name: str = None, file_name: str = None,
               life_time_in_hours: int = 4, comment: str = "Created via API",
               browser_name: str = "chrome") -> Tuple[Status, Union[dict, Exception]]:
        """
        Create token
        @param project_id Project Id
        @param project_name Project Name
        @param scope scope
        @param file_name File name
        @param life_time_in_hours Token lifetime in hours
        @param comment comment associated with the token
        @param browser_name: Browser name; allowed values: chrome, firefox, safari, edge
        @returns Tuple of Status, token json or Exception
        @raises Exception in case of failure
        """
        try:
            if project_id is None and project_name is None:
                raise CredMgrException("Project ID or Project Name must be specified")
            session = SessionHelper(url=f"https://{self.host}/", cookie_name=self.cookie_name,
                                    wait_timeout=self.wait_timeout, wait_interval=self.wait_interval)
            cookie = session.login(browser_name=browser_name)
            self.tokens_api.api_client.cookie = cookie
            tokens = self.tokens_api.tokens_create_post(project_id=project_id, project_name=project_name, scope=scope,
                                                        lifetime=life_time_in_hours, comment=comment)
            tokens_json = tokens.data[0].to_dict()
            if file_name is not None:
                with atomic_write(file_name, overwrite=True) as f:
                    json.dump(tokens_json, f)
            return Status.OK, tokens_json
        except Exception as e:
            return Status.FAILURE, e

    def refresh(self, scope: str, refresh_token: str, file_name: str = None, project_id: str = None,
                project_name: str = None) -> Tuple[Status, dict]:
        """
        Refresh token
        @param project_id Project Id
        @param project_name Project Name
        @param scope scope
        @param refresh_token refresh token
        @param file_name File name
        @returns Tuple of Status, dictionary {id_token/error, refresh_token, created_at}. In case of failure, id token would be None
        @raises Exception in case of failure
        """
        try:
            if project_id is None and project_name is None:
                raise CredMgrException("Project ID or Project Name must be specified")
            body = swagger_client.Request(refresh_token)
            tokens = self.tokens_api.tokens_refresh_post(body=body, project_id=project_id, project_name=project_name,
                                                         scope=scope)

            tokens_json = tokens.data[0].to_dict()
            if file_name is not None:
                with atomic_write(file_name, overwrite=True) as f:
                    json.dump(tokens_json, f)
            return Status.OK, tokens_json
        except CredMgrException as e:
            message = str(e.body)
            tokens_json = {self.ERROR: e.body,
                           self.CREATED_AT: datetime.strftime(datetime.utcnow(), self.TIME_FORMAT)}
            if message is not None and self.REFRESH_TOKEN in message:
                refresh_token = message.split(f"{self.REFRESH_TOKEN}:")[1]
                refresh_token = refresh_token.strip()
                refresh_token = refresh_token.strip("\"")
                refresh_token = refresh_token.strip("\n")
                tokens_json[self.REFRESH_TOKEN] = refresh_token
            return Status.FAILURE, tokens_json

    def revoke(self, identity_token: str, refresh_token: str = None, token_hash: str = None,
               token_type: TokenType = TokenType.Refresh) -> Tuple[Status, Any]:
        """
        Revoke token
        @param refresh_token refresh token
        @param identity_token identity token
        @param token_hash token hash for the identity token being revoked
        @param token_type token type
        @returns response
        @raises Exception in case of failure
        """
        try:
            if identity_token is None:
                raise CredMgrException(f"Identity Token is required")
            if refresh_token is None and token_type == TokenType.Refresh:
                raise CredMgrException(f"Refresh Token is required when revoking a refresh token")

            # Set the tokens
            self.__set_tokens(token=identity_token)

            if token_type == TokenType.Refresh:
                token = refresh_token
            else:
                token = token_hash

            body = swagger_client.TokenPost(type=str(token_type).lower(), token=token)
            self.tokens_api.tokens_revokes_post(body=body)

            return Status.OK, None
        except CredMgrException as e:
            return Status.FAILURE, e.body

    def clear_token_cache(self, *, file_name: str) -> Tuple[Status, Any]:
        """
        Clear cached token
        @param file_name name of the file containing the cached token
        @return STATUS.OK for success, STATUS.FAILURE and exception in case of failure
        """
        try:
            with open(file_name, 'r') as stream:
                token_data = json.loads(stream.read())
            if self.ID_TOKEN in token_data:
                token_data.pop(self.ID_TOKEN)
            with atomic_write(file_name, overwrite=True) as f:
                json.dump(token_data, f)
        except Exception as e:
            return Status.FAILURE, e
        return Status.OK, None

    def certs_get(self) -> Tuple[Status, Any]:
        """
        Return certificates
        """
        try:
            certs = self.default_api.certs_get()
            return Status.OK, certs
        except CredMgrException as e:
            return Status.FAILURE, e.body

    def version_get(self) -> Tuple[Status, Any]:
        """
        Return Version
        """
        try:
            version = self.version_api.version_get()
            return Status.OK, version
        except CredMgrException as e:
            return Status.FAILURE, e.body

    def tokens(self, *, token: str, project_id: str = None, expires: str = None, states: List[TokenState] = None,
               limit: int = 200, offset: int = 0,
               token_hash: str = None,) -> Tuple[Status, Union[Exception, List[Token]]]:
        """
        Return list of tokens issued to a user
        @return list of tokens
        """
        if token is None:
            return Status.INVALID_ARGUMENTS, CredMgrException(f"Token {token} must be specified")

        try:
            # Set the tokens
            self.__set_tokens(token=token)

            if expires is not None:
                expiry_time = datetime.strptime(expires, self.TIME_FORMAT)
            else:
                expiry_time = None

            tokens = self.tokens_api.tokens_get(states=TokenState.state_list_to_str_list(states=states), limit=limit,
                                                offset=offset, token_hash=token_hash, project_id=project_id,
                                                expires=expiry_time)

            return Status.OK, tokens.data if tokens.data is not None else []
        except Exception as e:
            return Status.FAILURE, e

    def token_revoke_list(self, *, project_id: str) -> Tuple[Status, Union[Exception, List[str]]]:
        """
        Return list of revoked tokens for a user
        @return list of revoked tokens
        """
        try:
            if project_id is None:
                return Status.INVALID_ARGUMENTS, CredMgrException(f"Project Id {project_id} must be specified")

            tokens = self.tokens_api.tokens_revoke_list_get(project_id=project_id)

            return Status.OK, tokens.data if tokens.data is not None else []
        except Exception as e:
            return Status.FAILURE, e

    def validate(self, *, token: str) -> Tuple[Status, Union[Exception, DecodedToken]]:
        """
        Validate a provided token and return decoded token
        @return status and decoded token
        """
        if token is None:
            return Status.INVALID_ARGUMENTS, CredMgrException(f"Token {token} must be specified")

        try:
            token_post = TokenPost(type="identity", token=token)
            slices = self.tokens_api.tokens_validate_post(token_post)

            return Status.OK, slices.data if slices.data is not None else []
        except Exception as e:
            return Status.FAILURE, e


from fabric_cm.credmgr.credmgr_proxy import CredmgrProxy

if __name__ == '__main__':
    cm = CredmgrProxy(credmgr_host="alpha-2.fabric-testbed.net", cookie_name="fabric-service-alpha")
    # By default; opens Chrome
    status, tokens = cm.create(project_name="CF TEST", comment="Create via Chrome")
    print(f"Token generated via Chrome: {tokens}")

    # By default; opens Chrome
    status, tokens = cm.create(project_name="CF TEST", browser_name="firefox", comment="Create via Firefox")
    print(f"Token generated via Firefox: {tokens}")

    # By default; opens Chrome
    status, tokens = cm.create(project_name="CF TEST", browser_name="safari", comment="Create via Safari")
    print(f"Token generated via Safari: {tokens}")






# coding: utf-8

"""
    Fabric Credential Manager API

    This is Fabric Credential Manager API  # noqa: E501

    OpenAPI spec version: 1.0.2
    Contact: kthare10@unc.edu
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Token(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'token_hash': 'str',
        'created_at': 'str',
        'expires_at': 'str',
        'state': 'str',
        'created_from': 'str',
        'comment': 'str',
        'id_token': 'str',
        'refresh_token': 'str'
    }

    attribute_map = {
        'token_hash': 'token_hash',
        'created_at': 'created_at',
        'expires_at': 'expires_at',
        'state': 'state',
        'created_from': 'created_from',
        'comment': 'comment',
        'id_token': 'id_token',
        'refresh_token': 'refresh_token'
    }

    def __init__(self, token_hash=None, created_at=None, expires_at=None, state=None, created_from=None, comment=None, id_token=None, refresh_token=None):  # noqa: E501
        """Token - a model defined in Swagger"""  # noqa: E501
        self._token_hash = None
        self._created_at = None
        self._expires_at = None
        self._state = None
        self._created_from = None
        self._comment = None
        self._id_token = None
        self._refresh_token = None
        self.discriminator = None
        self.token_hash = token_hash
        self.created_at = created_at
        self.expires_at = expires_at
        self.state = state
        self.created_from = created_from
        if comment is not None:
            self.comment = comment
        if id_token is not None:
            self.id_token = id_token
        if refresh_token is not None:
            self.refresh_token = refresh_token

    @property
    def token_hash(self):
        """Gets the token_hash of this Token.  # noqa: E501

        Identity Token SHA256 Hash  # noqa: E501

        :return: The token_hash of this Token.  # noqa: E501
        :rtype: str
        """
        return self._token_hash

    @token_hash.setter
    def token_hash(self, token_hash):
        """Sets the token_hash of this Token.

        Identity Token SHA256 Hash  # noqa: E501

        :param token_hash: The token_hash of this Token.  # noqa: E501
        :type: str
        """
        if token_hash is None:
            raise ValueError("Invalid value for `token_hash`, must not be `None`")  # noqa: E501

        self._token_hash = token_hash

    @property
    def created_at(self):
        """Gets the created_at of this Token.  # noqa: E501

        Token creation time  # noqa: E501

        :return: The created_at of this Token.  # noqa: E501
        :rtype: str
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this Token.

        Token creation time  # noqa: E501

        :param created_at: The created_at of this Token.  # noqa: E501
        :type: str
        """
        if created_at is None:
            raise ValueError("Invalid value for `created_at`, must not be `None`")  # noqa: E501

        self._created_at = created_at

    @property
    def expires_at(self):
        """Gets the expires_at of this Token.  # noqa: E501

        Token expiry time  # noqa: E501

        :return: The expires_at of this Token.  # noqa: E501
        :rtype: str
        """
        return self._expires_at

    @expires_at.setter
    def expires_at(self, expires_at):
        """Sets the expires_at of this Token.

        Token expiry time  # noqa: E501

        :param expires_at: The expires_at of this Token.  # noqa: E501
        :type: str
        """
        if expires_at is None:
            raise ValueError("Invalid value for `expires_at`, must not be `None`")  # noqa: E501

        self._expires_at = expires_at

    @property
    def state(self):
        """Gets the state of this Token.  # noqa: E501

        Token state  # noqa: E501

        :return: The state of this Token.  # noqa: E501
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this Token.

        Token state  # noqa: E501

        :param state: The state of this Token.  # noqa: E501
        :type: str
        """
        if state is None:
            raise ValueError("Invalid value for `state`, must not be `None`")  # noqa: E501
        allowed_values = ["Nascent", "Valid", "Refreshed", "Revoked", "Expired"]  # noqa: E501
        if state not in allowed_values:
            raise ValueError(
                "Invalid value for `state` ({0}), must be one of {1}"  # noqa: E501
                .format(state, allowed_values)
            )

        self._state = state

    @property
    def created_from(self):
        """Gets the created_from of this Token.  # noqa: E501

        Remote IP from where the token create request was received  # noqa: E501

        :return: The created_from of this Token.  # noqa: E501
        :rtype: str
        """
        return self._created_from

    @created_from.setter
    def created_from(self, created_from):
        """Sets the created_from of this Token.

        Remote IP from where the token create request was received  # noqa: E501

        :param created_from: The created_from of this Token.  # noqa: E501
        :type: str
        """
        if created_from is None:
            raise ValueError("Invalid value for `created_from`, must not be `None`")  # noqa: E501

        self._created_from = created_from

    @property
    def comment(self):
        """Gets the comment of this Token.  # noqa: E501

        Comment provided at creation  # noqa: E501

        :return: The comment of this Token.  # noqa: E501
        :rtype: str
        """
        return self._comment

    @comment.setter
    def comment(self, comment):
        """Sets the comment of this Token.

        Comment provided at creation  # noqa: E501

        :param comment: The comment of this Token.  # noqa: E501
        :type: str
        """

        self._comment = comment

    @property
    def id_token(self):
        """Gets the id_token of this Token.  # noqa: E501

        Identity Token  # noqa: E501

        :return: The id_token of this Token.  # noqa: E501
        :rtype: str
        """
        return self._id_token

    @id_token.setter
    def id_token(self, id_token):
        """Sets the id_token of this Token.

        Identity Token  # noqa: E501

        :param id_token: The id_token of this Token.  # noqa: E501
        :type: str
        """

        self._id_token = id_token

    @property
    def refresh_token(self):
        """Gets the refresh_token of this Token.  # noqa: E501

        Refresh Token  # noqa: E501

        :return: The refresh_token of this Token.  # noqa: E501
        :rtype: str
        """
        return self._refresh_token

    @refresh_token.setter
    def refresh_token(self, refresh_token):
        """Sets the refresh_token of this Token.

        Refresh Token  # noqa: E501

        :param refresh_token: The refresh_token of this Token.  # noqa: E501
        :type: str
        """

        self._refresh_token = refresh_token

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Token, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Token):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
